﻿using UnityEngine;
using System.Collections;


/// サンプルなので、移動処理はきっちり作っていません。

public class DogController : MonoBehaviour {

	public GameObject	TargetObject;		/// 目標位置
	public float		Jump = 20.0f;		/// エージェントのジャンプ力
	NavMeshAgent 		m_navMeshAgent;		/// NavMeshAgent

	/// オフメッシュリンク移動制御用構造体
	struct OffMeshLinkMove { 
		public bool		isMoveOnTheOffMeshLink;	/// オフメッシュリンクを移動中か
		public float	moveTime;				/// オフメッシュリンク間移動にかける時間
		public float	nowDisitance;			/// オフメッシュリンク間移動の現時点での移動距離
		public float	distanceOfOffMeshLinks;	/// オフメッシュリンクの距離
		public float	moveSpeed;				/// エージェントの実移動速度
		public Vector3	startPos;				/// オフメッシュリンクの始点
		public Vector3	endPos;					/// オフメッシュリンクの終点
	};
	OffMeshLinkMove		m_offMeshLinkMove;	/// オフメッシュリンク移動制御用構造体


	// Use this for initialization
	void Start () {

		// NavMeshAgentコンポーネントを取得
		m_navMeshAgent = GetComponent<NavMeshAgent>();

		// オフメッシュリンクを自動で飛び越えないようにする
		m_navMeshAgent.autoTraverseOffMeshLink = false;

		// オフメッシュリンク移動制御用構造体を初期化
		InitOffMeshLinkMove ();
	}

	// Update is called once per frame
	void Update () {


		// NavMeshが準備できているなら
		if(m_navMeshAgent.pathStatus != NavMeshPathStatus.PathInvalid) {
			
			// NavMeshAgentに目的地をセット
			m_navMeshAgent.SetDestination(TargetObject.transform.position);



			// オフメッシュリンクに辿り着いているなら
			if (m_navMeshAgent.isOnOffMeshLink){

				// オフメッシュリンクの情報を取得
				OffMeshLinkData data = m_navMeshAgent.currentOffMeshLinkData;	

				// オフメッシュリンク移動の初期化処理---------------------------
				if (m_offMeshLinkMove.distanceOfOffMeshLinks == 0) {
					// オフメッシュリンクの距離を求める
					m_offMeshLinkMove.distanceOfOffMeshLinks = Vector3.Distance (data.startPos, data.endPos);

					// 目的地の方向を向かせる
					transform.LookAt (new Vector3 (data.endPos.x, transform.position.y, data.endPos.z));	
				}
				// オフメッシュリンク移動の初期化処理---------------------------


				// 移動処理------------------------------------------------
				// 現時点での移動距離を求める
				m_offMeshLinkMove.nowDisitance += Time.deltaTime * m_offMeshLinkMove.distanceOfOffMeshLinks / m_offMeshLinkMove.moveTime;

				// 移動開始地点と目標地点を指定して移動　3つめの引数では現時点での移動開始地点からの移動距離を指定する
				Vector3 pos = Vector3.MoveTowards (data.startPos, data.endPos, m_offMeshLinkMove.nowDisitance);

				// Sin波を使って放物線を描く
				float degree = (m_offMeshLinkMove.nowDisitance / m_offMeshLinkMove.distanceOfOffMeshLinks * 180.0f);
				pos.y = data.startPos.y + (Mathf.Sin (Mathf.Deg2Rad * degree) * Jump);

				// 下降を始めた時に終点よりを防ぐ
				if (degree > 90.0f) {
					if (pos.y < data.endPos.y) {
						pos.y = data.endPos.y;
					}
				}

				// 座標を反映
				transform.position = pos;
				// 移動処理------------------------------------------------


				// 移動完了判定---------------------------------------------
				if ((transform.position.x == data.endPos.x) && (transform.position.z == data.endPos.z)) {
					m_navMeshAgent.CompleteOffMeshLink ();	// オフメッシュリンクの移動を完了させたことをAgentに通知
					InitOffMeshLinkMove ();					// オフメッシュリンク移動制御用構造体を初期化
				}
				// 移動完了判定---------------------------------------------
			}

		}
	}


	// オフメッシュリンク移動制御用構造体を初期化
	void InitOffMeshLinkMove () {
		m_offMeshLinkMove.isMoveOnTheOffMeshLink = false;
		m_offMeshLinkMove.moveTime = 1.0f;
		m_offMeshLinkMove.nowDisitance = 0;
		m_offMeshLinkMove.distanceOfOffMeshLinks = 0;
		m_offMeshLinkMove.moveSpeed = 0;
	}
}
