﻿using UnityEngine;
using System.Collections;

/// <summary>
/// 3人称視点のカメラ
/// </summary>
public class ThirdPersonCamera : MonoBehaviour {

	public GameObject	Target;					/// 目標物
	public float		Smoothing = 0.1f;		/// カメラの動きの滑らかさ（数字が大きいほど、カチッとした動きになる）
	public float		Height = 1.0f;			/// 目標物からのカメラの高さ
	public float		Length = 20.0f;			/// カメラと目標物の距離

	// Use this for initialization
	void Start () {
	
	}
	
	// Update is called once per frame
	void Update () {
		
		// 目標物の後ろにカメラを配置
		Vector3 targetCamPos = Target.transform.position + (Target.transform.TransformDirection (Vector3.back) * Length);

		// 目標物の後ろに向かって滑らかに移動
		transform.position =  Vector3.Lerp(transform.position, targetCamPos, Smoothing);

		// カメラの高さを調節
		Vector3 pos = transform.position;	// 現在の位置を取得して
		pos.y += Height;					// 目標物からのカメラの高さを加算
		transform.position = pos;			// カメラに値を反映

		// 目標物の方向を見る
		transform.LookAt (Target.transform);

	}
}
